"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
const fanout_1 = require("thingies/es2020/fanout");
const nodes_1 = require("../../nodes");
const nodes_2 = require("./nodes");
const Emitter_1 = require("../../../util/events/Emitter");
const PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
class ModelApi {
    constructor(model) {
        this.model = model;
        this.next = 0;
        this.changes = new fanout_1.FanOut();
        this.queuedChanges = undefined;
        this.queueChange = (changeType) => {
            this.changes.emit(changeType);
            let changesQueued = this.queuedChanges;
            if (changesQueued) {
                changesQueued.add(changeType);
                return;
            }
            changesQueued = this.queuedChanges = new Set();
            changesQueued.add(changeType);
            queueMicrotask(() => {
                let changes = this.queuedChanges || new Set();
                this.queuedChanges = undefined;
                const et = this.et;
                if (changes.has(2))
                    changes = new Set([2]);
                if (et)
                    et.emit(new CustomEvent('change', { detail: changes }));
            });
        };
        this.et = new Emitter_1.Emitter();
        this.subscribe = (callback) => {
            const listener = () => callback();
            this.events.on('change', listener);
            return () => this.events.off('change', listener);
        };
        this.getSnapshot = () => this.view();
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
        this.model.onchange = this.queueChange;
    }
    get events() {
        return this.et;
    }
    wrap(node) {
        if (node instanceof nodes_1.ValNode)
            return node.api || (node.api = new nodes_2.ValApi(node, this));
        else if (node instanceof nodes_1.StrNode)
            return node.api || (node.api = new nodes_2.StrApi(node, this));
        else if (node instanceof nodes_1.BinNode)
            return node.api || (node.api = new nodes_2.BinApi(node, this));
        else if (node instanceof nodes_1.ArrNode)
            return node.api || (node.api = new nodes_2.ArrApi(node, this));
        else if (node instanceof nodes_1.ObjNode)
            return node.api || (node.api = new nodes_2.ObjApi(node, this));
        else if (node instanceof nodes_1.ConNode)
            return node.api || (node.api = new nodes_2.ConApi(node, this));
        else if (node instanceof nodes_1.VecNode)
            return node.api || (node.api = new nodes_2.VecApi(node, this));
        else
            throw new Error('UNKNOWN_NODE');
    }
    get r() {
        return new nodes_2.ValApi(this.model.root, this);
    }
    get node() {
        return this.r.get();
    }
    in(path) {
        return this.r.in(path);
    }
    find(path) {
        return this.node.find(path);
    }
    val(path) {
        return this.node.val(path);
    }
    vec(path) {
        return this.node.tup(path);
    }
    str(path) {
        return this.node.str(path);
    }
    bin(path) {
        return this.node.bin(path);
    }
    arr(path) {
        return this.node.arr(path);
    }
    obj(path) {
        return this.node.obj(path);
    }
    const(path) {
        return this.node.const(path);
    }
    root(json) {
        const builder = this.builder;
        builder.root(builder.json(json));
        this.apply();
        return this;
    }
    apply() {
        const ops = this.builder.patch.ops;
        const length = ops.length;
        const model = this.model;
        for (let i = this.next; i < length; i++)
            model.applyOperation(ops[i]);
        this.next = length;
        model.tick++;
        model.onchange?.(1);
    }
    advance() {
        this.next = this.builder.patch.ops.length;
        const model = this.model;
        model.tick++;
        model.onchange?.(1);
    }
    view() {
        return this.model.view();
    }
    flush() {
        const patch = this.builder.flush();
        this.next = 0;
        const event = new CustomEvent('flush', { detail: patch });
        this.events.emit(event);
        return patch;
    }
}
exports.ModelApi = ModelApi;
