import { FanOut } from 'thingies/es2020/fanout';
import { Emitter } from '../../../util/events/Emitter';
class ChangesFanOut extends FanOut {
    api;
    _v = undefined;
    _u = undefined;
    constructor(api) {
        super();
        this.api = api;
    }
    listen(listener) {
        if (!this.listeners.size) {
            const api = this.api;
            this._v = api.view();
            this._u = api.api.changes.listen(() => {
                const view = api.view();
                if (view !== this._v) {
                    this._v = view;
                    this.emit(view);
                }
            });
        }
        const unsubscribe = super.listen(listener);
        return () => {
            unsubscribe();
            if (!this.listeners.size) {
                this._u?.();
                this._u = this._v = undefined;
            }
        };
    }
    dispose() {
        this.listeners.clear();
        this._u?.();
        this._u = this._v = undefined;
    }
}
export class NodeEvents extends Emitter {
    api;
    changes;
    constructor(api) {
        super();
        this.api = api;
        this.changes = new ChangesFanOut(api);
    }
    viewSubs = new Set();
    _view = undefined;
    onModelChange = () => {
        const _view = this._view;
        const view = this.api.node.view();
        const viewHasChanged = _view !== view;
        this._view = view;
        if (viewHasChanged)
            this.emit(new CustomEvent('view'));
    };
    setupViewEvents() {
        this._view = this.api.node.view();
        this.api.api.events.on('change', this.onModelChange);
    }
    on(type, listener, options) {
        if (type === 'view')
            this.viewSubs.add(listener);
        const shouldSubscribeToModelChanges = this.viewSubs.size === 1;
        if (shouldSubscribeToModelChanges)
            this.setupViewEvents();
        super.on(type, listener, options);
    }
    off(type, listener, options) {
        if (type === 'view')
            this.viewSubs.delete(listener);
        const shouldUnsubscribeFromModelChanges = this.viewSubs.size === 1;
        if (shouldUnsubscribeFromModelChanges)
            this.api.api.events.off('change', this.onModelChange);
        super.off(type, listener, options);
    }
    onDelete() {
        this.changes.dispose();
    }
    subscribe = (callback) => {
        return this.changes.listen(() => callback());
    };
    getSnapshot = () => this.api.view();
}
