"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeEvents = void 0;
const fanout_1 = require("thingies/es2020/fanout");
const Emitter_1 = require("../../../util/events/Emitter");
class ChangesFanOut extends fanout_1.FanOut {
    constructor(api) {
        super();
        this.api = api;
        this._v = undefined;
        this._u = undefined;
    }
    listen(listener) {
        if (!this.listeners.size) {
            const api = this.api;
            this._v = api.view();
            this._u = api.api.changes.listen(() => {
                const view = api.view();
                if (view !== this._v) {
                    this._v = view;
                    this.emit(view);
                }
            });
        }
        const unsubscribe = super.listen(listener);
        return () => {
            var _a;
            unsubscribe();
            if (!this.listeners.size) {
                (_a = this._u) === null || _a === void 0 ? void 0 : _a.call(this);
                this._u = this._v = undefined;
            }
        };
    }
    dispose() {
        var _a;
        this.listeners.clear();
        (_a = this._u) === null || _a === void 0 ? void 0 : _a.call(this);
        this._u = this._v = undefined;
    }
}
class NodeEvents extends Emitter_1.Emitter {
    constructor(api) {
        super();
        this.api = api;
        this.viewSubs = new Set();
        this._view = undefined;
        this.onModelChange = () => {
            const _view = this._view;
            const view = this.api.node.view();
            const viewHasChanged = _view !== view;
            this._view = view;
            if (viewHasChanged)
                this.emit(new CustomEvent('view'));
        };
        this.subscribe = (callback) => {
            return this.changes.listen(() => callback());
        };
        this.getSnapshot = () => this.api.view();
        this.changes = new ChangesFanOut(api);
    }
    setupViewEvents() {
        this._view = this.api.node.view();
        this.api.api.events.on('change', this.onModelChange);
    }
    on(type, listener, options) {
        if (type === 'view')
            this.viewSubs.add(listener);
        const shouldSubscribeToModelChanges = this.viewSubs.size === 1;
        if (shouldSubscribeToModelChanges)
            this.setupViewEvents();
        super.on(type, listener, options);
    }
    off(type, listener, options) {
        if (type === 'view')
            this.viewSubs.delete(listener);
        const shouldUnsubscribeFromModelChanges = this.viewSubs.size === 1;
        if (shouldUnsubscribeFromModelChanges)
            this.api.api.events.off('change', this.onModelChange);
        super.off(type, listener, options);
    }
    onDelete() {
        this.changes.dispose();
    }
}
exports.NodeEvents = NodeEvents;
